searchData={"items":[{"type":"module","title":"DVR","doc":"Documentation for Dvr.","ref":"DVR.html"},{"type":"function","title":"DVR.calculate_id/0","doc":"","ref":"DVR.html#calculate_id/0"},{"type":"function","title":"DVR.record/2","doc":"","ref":"DVR.html#record/2"},{"type":"function","title":"DVR.record/3","doc":"","ref":"DVR.html#record/3"},{"type":"function","title":"DVR.replay/1","doc":"","ref":"DVR.html#replay/1"},{"type":"function","title":"DVR.search/1","doc":"","ref":"DVR.html#search/1"},{"type":"type","title":"DVR.payload/0","doc":"","ref":"DVR.html#t:payload/0"},{"type":"type","title":"DVR.replay_id/0","doc":"","ref":"DVR.html#t:replay_id/0"},{"type":"type","title":"DVR.topics/0","doc":"","ref":"DVR.html#t:topics/0"},{"type":"module","title":"DVR.AbsintheChannel","doc":"An Absinthe channel mixin that handles replaying of recorded messages.","ref":"DVR.AbsintheChannel.html"},{"type":"function","title":"DVR.AbsintheChannel.handle_in/3","doc":"","ref":"DVR.AbsintheChannel.html#handle_in/3"},{"type":"module","title":"DVR.Channel","doc":"A Phoenix channel mixin that handles replaying of recorded messages","ref":"DVR.Channel.html"},{"type":"module","title":"DVR.Cleanup","doc":"A process to periodically cleanup stored messages that are older than the configured ttl.","ref":"DVR.Cleanup.html"},{"type":"function","title":"DVR.Cleanup.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\n`arg` is passed as the argument to `Task.start_link/1` in the `:start` field\nof the spec.\n\nFor more information, see the `Supervisor` module,\nthe `Supervisor.child_spec/2` function and the `t:Supervisor.child_spec/0` type.","ref":"DVR.Cleanup.html#child_spec/1"},{"type":"function","title":"DVR.Cleanup.cleanup/1","doc":"","ref":"DVR.Cleanup.html#cleanup/1"},{"type":"function","title":"DVR.Cleanup.run/0","doc":"","ref":"DVR.Cleanup.html#run/0"},{"type":"function","title":"DVR.Cleanup.run/1","doc":"","ref":"DVR.Cleanup.html#run/1"},{"type":"function","title":"DVR.Cleanup.start_link/1","doc":"","ref":"DVR.Cleanup.html#start_link/1"},{"type":"module","title":"DVR.Store","doc":"Mnesia table for replay storage.","ref":"DVR.Store.html"},{"type":"function","title":"DVR.Store.copy_store/0","doc":"Mnesiac will call this method to copy the table","ref":"DVR.Store.html#copy_store/0"},{"type":"macro","title":"DVR.Store.dvr_message/1","doc":"","ref":"DVR.Store.html#dvr_message/1"},{"type":"macro","title":"DVR.Store.dvr_message/2","doc":"","ref":"DVR.Store.html#dvr_message/2"},{"type":"function","title":"DVR.Store.init_store/0","doc":"Mnesiac will call this method to initialize the table","ref":"DVR.Store.html#init_store/0"},{"type":"type","title":"DVR.Store.dvr_message/0","doc":"","ref":"DVR.Store.html#t:dvr_message/0"},{"type":"extras","title":"DVR","doc":"# DVR\n\n*Record and replay your Phoenix channels*\n\n![Hex.pm](https://img.shields.io/hexpm/v/dvr.svg)\n![Hex.pm licence](https://img.shields.io/hexpm/l/dvr.svg)\n![Build Status)](https://img.shields.io/github/actions/workflow/status/athal7/dvr/ci.yml?branch=main)\n\n**Documentation can be found at [https://hexdocs.pm/dvr](https://hexdocs.pm/dvr).**\n\nDVR gives you the ability to resend channel messages from your Phoenix server, based on a client-supplied id for the last seen message. Unlike the [example mentioned in the Phoenix Docs](https://hexdocs.pm/phoenix/channels.html#resending-server-messages), this implementation utilizes [mnesia](http://erlang.org/doc/man/mnesia.html), as opposed to an external database backend.","ref":"readme.html"},{"type":"extras","title":"Installation - DVR","doc":"The package can be installed by adding `dvr` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:dvr, \"~> 1.1.0\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Configuration - DVR","doc":"#","ref":"readme.html#configuration"},{"type":"extras","title":"Mnesia - DVR","doc":"The mnesia table can be easily setup across your cluster by utilizing the [mnesiac](https://github.com/beardedeagle/mnesiac) package.\n\n```elixir\nconfig :mnesiac, stores: [DVR.Store]\n```\n\nOr you can configure the mnesia table on your own at startup:\n\n```elixir\nDVR.Store.init_store()\nDVR.Store.copy_store()\n```\n\n#","ref":"readme.html#mnesia"},{"type":"extras","title":"Cleanup - DVR","doc":"You probably want to cleanup the saved messages after a period of time, so as to not over-use your memory or disc capacity (based on the mnesia backend you choose). You can do so by adding the provided cleanup task to your supervision tree:\n\n```elixir\nchildren = [DVR.Cleanup]\nSupervisor.start_link(children, strategy: :one_for_one)\n```\n\nThe default interval is 1 minute, and the default ttl is 1 hour, but you can configure them as you desire:\n\n```elixir\nchildren = [{DVR.Cleanup, interval_seconds: 60 * 10, ttl_seconds: 60 * 60 * 24}]\nSupervisor.start_link(children, strategy: :one_for_one)\n```","ref":"readme.html#cleanup"},{"type":"extras","title":"Usage - DVR","doc":"#","ref":"readme.html#usage"},{"type":"extras","title":"Basic Usage - DVR","doc":"**Record a message**\n\n```elixir\n{:ok, id} = DVR.record(%{some: \"message\"}, ['some_topic'])\n```\n\n**Replay missed messages**\n\n```elixir\nid # last seen message id\n|> DVR.replay(['some_topic'])\n|> Stream.each(&send_to_client/1) # your implementation\n|> Stream.run()\n```\n\n**Check for a message by id**\n\n```elixir\n{:ok, id} = DVR.search(id)\n```\n\n#","ref":"readme.html#basic-usage"},{"type":"extras","title":"With Phoenix - DVR","doc":"In `channel.ex`\n\n```elixir\ndefmodule MyApp.Channel do\n  use Phoenix.Channel\n  use DVR.Channel\n\n  ...\n\n  def handle_in(\"new_msg\", msg, socket) do\n    case DVR.record(msg, [socket.topic]) do\n      {:ok, replay_id} ->\n        broadcast!(socket, socket.topic, Map.put(msg, :replay_id, replay_id))\n\n      err ->\n        Logger.error(\"Unable to add replayId to message\", error: err)\n        push(socket, socket.topic, msg)\n    end\n\n    {:noreply, socket}\n  end\nend\n```\n\nIn your client:\n\n```js\n...\n\nlet replayId // recovered from storage somewhere\n\nchannel.on(\"new_msg\", payload => {\n  lastMessageId = payload.replay_id\n})\n\nchannel.join()\n  .receive(\"ok\", resp => {\n    console.log(\"Joined successfully\", resp)\n    channel.push('replay', { replayId })\n  })\n  .receive(\"error\", resp => { console.log(\"Unable to join\", resp) })\n```\n\n#","ref":"readme.html#with-phoenix"},{"type":"extras","title":"With Absinthe - DVR","doc":"Make sure to add the `replayId` to your schema for the subscription type that you are publishing. Then you can record the message when resolving:\n\n```elixir\nobject :foo do\n  field(:bar, :string)\n  field(:baz, :string)\nend\n\nobject :foo_update do\n  field(:foo, :foo)\n  field(:replayId, :integer)\nend\n\nsubscription do\n  field :foo_updates, :foo_update do\n    config(fn _, _ -> {:ok, topic: \"*\"} end)\n\n    resolve(fn root, _, _ ->\n      {:ok, replay_id} = DVR.record(root, [foo_updates: \"*\"])\n      {:ok, Map.put(root, :replay_id, replay_id)}\n    end)\n  end\nend\n```\n\nFor now, you have to customize the entire set of channel / socket modules, since there's not yet a way to decorate the default channel:\n\nendpoint.ex\n\n```elixir\ndefmodule MyApp.Endpoint do\n  use Phoenix.Endpoint, otp_app: :web\n  use Absinthe.Phoenix.Endpoint\n\n  socket(\"/socket\", MyApp.UserSocket, websocket: true)\n  ...\n```\n\nsocket.ex\n\n```elixir\ndefmodule MyApp.UserSocket do\n  use Phoenix.Socket\n\n  def connect(_payload, socket), do: {:ok, socket}\n  def id(_socket), do: nil\n\n  channel(\n    \"__absinthe__:*\",\n    MyApp.AbsintheChannel,\n    assigns: %{__absinthe_schema__: MyApp.Schema}\n  )\n\n  defdelegate put_options(socket, opts), to: Absinthe.Phoenix.Socket\n  defdelegate put_schema(socket, schema), to: Absinthe.Phoenix.Socket\nend\n```\n\nchannel.ex\n\n```elixir\ndefmodule MyApp.Channel do\n  use Phoenix.Channel\n\n  defdelegate handle_in(event, payload, socket), to: DVR.AbsintheChannel\n  defdelegate join(channel, message, socket), to: Absinthe.Phoenix.Channel\nend\n```\n\nIn your client:\n\n```js\n...\n\nlet replayId // recovered from storage somewhere\n\nchannel.on(\"new_msg\", payload => {\n  // take the replayId from the relevant place in your schema\n  replayId = payload.replayId\n})\n\nchannel.join()\n  .receive(\"ok\", resp => {\n    console.log(\"Joined successfully\", resp)\n    const subscriptionId = resp.body.payload.response.subscriptionId\n    channel.push('replay', { replayId, subscriptionId })\n  })\n  .receive(\"error\", resp => { console.log(\"Unable to join\", resp) })\n```","ref":"readme.html#with-absinthe"}],"content_type":"text/markdown"}